/***************************************************************************
 *   Copyright (C) 2006-2011 by Julien Lemoine, Simon Viennot              *
 *   lapinot@tuxfamily.org                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef GLOPWIDGET_H
#define GLOPWIDGET_H

#include <QMainWindow>
#include <QWidget>
#include <QTextEdit>
#include "game.h"
#include "database.h"
#include "tracewidget.h"
#include "interfacewidget.h"

//graphical user interface classes
class QTimer;
class QLabel;
class QString;
class QTabWidget;
class QSpinBox;
class QCheckBox;
class QRadioButton;
class QPushButton;
class QSignalMapper;
class QLineEdit;
class QCloseEvent;
class QComboBox;
class QGroupBox;


//simple class to respond to double-click (used by the children tab)
class ClickTextEdit : public QTextEdit {
	Q_OBJECT
  public:
	ClickTextEdit(QWidget * parent = 0);

  protected:
	//reimplementation of double-click
    void mouseDoubleClickEvent( QMouseEvent * event );

  signals:
	void lineContent(QString);
};

//class for the main window
class GlopWidget : public QMainWindow {
	Q_OBJECT
  public:
	GlopWidget( QWidget *parent=0 );
  //main object representing the game
	Game *game;
	
  //graphical interface
	//tabs
	static QTabWidget *tabNodeWidget;
	static QTabWidget *tabWidget;
	QWidget *explore_tab;
	void setup_parameters_tab();
	void setup_compute_tab();
//	void setup_analysis_tab();
	void setup_test_tab();
	void setup_info_tab();
	
	//game parameters
	static QComboBox *box_game_choice, *box_traversal_choice;
	static vector<InterfaceWidget *> gameWidgets, nodeWidgets, traversalWidgets, commonWidgets;
	
	//general parameters
	int refreshTime;
	
	//analysis parameters widget
	/*QCheckBox *check_lives_max, *check_lives_min, *check_nimber_max, *check_nimber_min;
	QSpinBox *box_lives_max, *box_lives_min, *box_nimber_max, *box_nimber_min;*/
	
	//connexion mapping
	QSignalMapper *readText_Mapper;
	
	//tables to trace computation
	WidgetTrace *widget_trace;
	
	QPushButton *pause_button;
	
	//label to display the state of nodestore
	QLabel *nodeStoreState;
	
	//to print children
	QLineEdit *box_given_position_2;
	ClickTextEdit *textEdit_children;
	list<QString> positionCache;
	
	static QString glopVersionNumber;
	
  protected:
	//override QApplication virtual function closeEvent
	void closeEvent(QCloseEvent *event);
	
	bool computeRunning;
	bool pauseEnabled;
	bool noMoreMessage;	//used to avoid the message when closing during a computation
	
  public slots:
	//menu
	void read_text_file(int num);
	
	//start/stop computation
	void startComputation();
	void stop_computing();
	void pause_computing();
	
	void gameChoice(int);  //choose a different game in the parameter tab
	void selectPossibleNode(int);  //enable/disable nodes depending on the game category
	void traversalChoice(int);  //choose a different traversal in the parameter tab
	
  //trace computation related functions
	void activate_timer(bool);
	void trace_refresh(); //called on each timer's impulse
	void nodeStore_refresh();
	void wait_result();
	void displayExplorer();
	
  //test functions
	void printChildren();
	void nextChildren(QString);
	void previousChildren();
	
  signals:
	void isComputing(bool);	//specifies that the computation starts/stops
	void isChecking(bool);	//check computing only
	
  private:
	QTimer * impulse; //timer object with regular impulsion to refresh the graphical interface
};

#endif
